;*** Oddajnik za preizkus transponderja in DME - S53MV - 28/2/2004 ***

;*** Uporaba vrat PICa 16F84 ***
;prozenje/vklop oddajnika PTT=RA0
;izhod impulzov /PULSE=RA1
;BACKPLANE=RA2
;I2C vodilo: SCL=RA3, SDA=RA4 - zaenkrat neuporabljeno
;vhod prekinitev INT=RB0
;STROBE=RB1
;DATA=RB2
;CLOCK=RB3
;tipka DESNO=RB4
;tipka DOL=RB5
;tipka GOR=RB6
;tipka LEVO=RB7

;*** Uporaba RAMa ***
;0C     LCDBIT, IZVOR, LCDDEK
;0D	LCDDAT, CAKAJ, PRIKAZ
;0E	LCDDAT oscilator za BACKPLANE in utripanje stevilk 
;0F	TIPKE spomin prejsnjega stanja
;10,11,12,13	LCDDAT vsebina zaslona
;14	ODDAJA stevec zakasnitve

;20		zastavica vpisa/branja
;21,22,23,24	koda: 21=vzorec, 22=frekvenca, 23=izvor 24=ATC/DME
;25		zakasnitev oddaje 0-255 mikrosekund-1

;*** Zacetek programa, inicializacije ***
	LIST    P=16F84
	ORG	0
	CLRF	20	;pobrisi zastavico beri/vpisi EEPROM
	CALL	RMEM	;zacetne vrednosti spremenljivk
	CALL	INIPIC	;inicializacija vrat PICa
	GOTO	GLAVNI

;*** Prekinitveni program ***
	ORG	4
PREKIN	CALL	ODDAJA
	BSF	3,5	;postavi PAGE1
	BSF	1,0	;PS0
	BTFSC	22,0
	BCF	1,0
	BSF	1,1	;PS1
	BTFSC	22,1
	BCF	1,1
	BSF	1,2	;PS2
	BTFSC	22,2
	BCF	1,2
	BCF	3,5	;postavi PAGE0
	BCF	0B,1	;RESET INTF
	BCF	0B,2	;RESET T0IF
	RETFIE

;*** Izvori prekinitev *** (0C)
IZVOR	MOVLW	80
	MOVWF	0C
	BTFSC	23,0	;vkljuci INTE?
	BSF	0C,4
	BTFSC	23,1	;vkljuci T0IE?
	BSF	0C,5
	MOVF	0C,0
	MOVWF	0B
	RETURN

;*** Inicializacija vrat PICa ***
INIPIC	MOVLW   1A      ;zacetno stanje vrat A
	MOVWF   5
	MOVLW   0       ;zacetno stanje vrat B
	MOVWF   6
	BSF     3,5     ;postavi PAGE1
	MOVLW   0       ;port A: izhodi RA0-4
	MOVWF   5       ;vpisi TRISA
	MOVLW   0F1     ;port B: vhod RB0, izhodi RB1-3, vhodi RB4-7
	MOVWF   6       ;vpisi TRISB
	MOVLW	7	;OPTION: /RBPU, PS0,1,2
	MOVWF	1	
	BCF     3,5     ;postavi PAGE0
	RETURN

;*** Precitaj kodo iz EEPROMa ***
RMEM	MOVLW	21	;kazalec v RAMu
	MOVWF	4
	CLRF	9	;kazalec v EEPROMu
	CALL	RMEM1
	CALL	RMEM1
	CALL	RMEM1
	CALL	RMEM1
	CALL	RMEM1
	MOVLW	20	;kazalec v RAMu
	MOVWF	4
	RETURN
RMEM1	BSF	3,5	;postavi PAGE1 - precitaj bajt in povecaj stevca
	BSF	8,0	;EECON1 RD
	BCF	3,5	;postavi PAGE0
	MOVF	8,0
	MOVWF	0
	INCF	4,1	;povecaj stevca
	INCF	9,1
	RETURN

;*** Vpisi kodo v EEPROM ***
WMEM	MOVLW	21	;kazalec v RAMu
	MOVWF	4
	CLRF	9	;kazalec v EEPROMu
	CALL	WMEM1
	CALL	WMEM1
	CALL	WMEM1
	CALL	WMEM1
	CALL	WMEM1
	MOVLW	20	;kazalec v RAMu
	MOVWF	4
	RETURN
WMEM1	BSF	3,5	;postavi PAGE1 - primerjaj vsebino EEPROMa
	BSF	8,0	;EECON1 RD
	BCF	3,5	;postavi PAGE0
	MOVF	8,0
	SUBWF	0,0
	BTFSC	3,2
	GOTO	WMEM3
	MOVF	0,0	;razlika  - vpisi novo vsebino v EEPROM
	MOVWF	8	
	BSF	3,5	;postavi PAGE1
	BCF	8,3	;EECON1	WRERR reset
	BSF	8,2	;EECON1 WREN set
	MOVLW	55
	MOVWF	9	;EECON2 55
	MOVLW	0AA
	MOVWF	9	;EECON2	0AA
	BSF	8,1	;EECON1 WR
WMEM2	BTFSS	8,4	;EECON1 EEIF test
	GOTO	WMEM2
	BCF	8,4	;EECON1 EEIF reset
	BCF	8,2	;EECON1 WREN reset
	BCF	3,5	;postavi PAGE0
WMEM3	INCF	4,1	;povecaj stevca
	INCF	9,1
	RETURN

;*** Beri/vpisi EEPROM ***
RWMEM	MOVF	20,1	;zastavica postavljena?
	BTFSC	3,2
	RETURN
	BCF	0B,7	;prepreci prekinitve
	BTFSC	20,7	;beri EEPROM?
	CALL	RMEM
	BTFSS	20,7	;vpisi EEPROM?
	CALL	WMEM
	BSF	0B,7	;omogoci prekinitve
	CLRF	20	;pobrisi zastavico
	RETURN

;*** Glavna zanka programa ***
GLAVNI	CALL	RWMEM	;beri/vpisi EEPROM
	CALL	MEJE	;preveri meje spremenljivk
	CALL	PRIKAZ	;priprava podatkov prikaza
	CALL	LCDDAT	;poslji podatke v pomikalni register
	CALL	CAKAJ	;cakanje 10ms
	CALL	TIPKE	;izvedi ukaz s tipk
	CALL	IZVOR	;izvor prekinitev
	GOTO	GLAVNI

;*** Prozenje in oddaja vprasanja *** (14)
ODDAJA	BSF	14,7	;prepisi zacetno vrednost stevca (25)>(14)
	BTFSS	25,7
	BCF	14,7
	BSF	14,6
	BTFSS	25,6
	BCF	14,6
	BSF	14,5
	BTFSS	25,5
	BCF	14,5
	BSF	14,4
	BTFSS	25,4
	BCF	14,4
	BSF	14,3
	BTFSS	25,3
	BCF	14,3
	BSF	14,2
	BTFSS	25,2
	BCF	14,2
	BSF	14,1
	BTFSS	25,1
	BCF	14,1
	BSF	14,0
	BTFSS	25,0
	BCF	14,0
	BSF	5,0	;prozenje/vklop oddaje
ODDAJA1	NOP	;zakasnitev 0-255 mikrosekund
	NOP
	DECFSZ	14,1
	GOTO	ODDAJA1
	BTFSS	24,0	;izberi vrsto oddaje?
	GOTO	ATCODD
	GOTO	DMEODD

;*** Prozenje in oddaja ATC vprasanja pri taktu 20MHz ***
ATCODD	BCF	5,1	;0us zacetni impulz
	NOP
	NOP
	NOP
	BSF	5,1
	NOP	;1us
	NOP
	NOP
	NOP
	BTFSC	21,0	;impulz OMNI antene
	BCF	5,1	;2us
	NOP
	NOP
	NOP
	BSF	5,1
	NOP	;3us
	NOP
	NOP
	NOP
	NOP
	NOP	;4us
	NOP
	NOP
	NOP
	NOP
	NOP	;5us
	NOP
	NOP
	NOP
	NOP
	NOP	;6us
	NOP
	NOP
	NOP
	NOP
	NOP	;7us
	NOP
	NOP
	NOP
	BTFSC	21,1	;vprasanje MODE A
	BCF	5,1	;8us
	NOP
	NOP
	NOP
	BSF	5,1
	NOP	;9us
	NOP
	NOP
	NOP
	NOP
	NOP	;10us
	NOP
	NOP
	NOP
	NOP
	NOP	;11us
	NOP
	NOP
	NOP
	NOP
	NOP	;12us
	NOP
	NOP
	NOP
	NOP
	NOP	;13us
	NOP
	NOP
	NOP
	NOP
	NOP	;14us
	NOP
	NOP
	NOP
	NOP
	NOP	;15us
	NOP
	NOP
	NOP
	NOP
	NOP	;16us
	NOP
	NOP
	NOP
	NOP
	NOP	;17us
	NOP
	NOP
	NOP
	NOP
	NOP	;18us
	NOP
	NOP
	NOP
	NOP
	NOP	;19us
	NOP
	NOP
	NOP
	NOP
	NOP	;20us
	NOP
	NOP
	NOP
	BTFSC	21,2	;vprasanje MODE C
	BCF	5,1	;21us
	NOP
	NOP
	NOP
	BSF	5,1
	NOP
	NOP
	NOP
	NOP
	BCF	5,0	;prozenje/izklop oddaje
	RETURN

;*** Prozenje in oddaja DME vprasanja pri taktu 20MHz ***
DMEODD	BCF	5,1	;0us zacetni impulz
	NOP
	NOP
	NOP
	NOP
	NOP	;1us
	NOP
	NOP
	NOP
	NOP
	NOP	;2us
	NOP
	NOP
	NOP
	NOP
	NOP	;3us
	NOP
	BSF	5,1	;3.4us
	NOP
	NOP
	NOP	;4us
	NOP
	NOP
	NOP
	NOP
	NOP	;5us
	NOP
	NOP
	NOP
	NOP
	NOP	;6us
	NOP
	NOP
	NOP
	NOP
	NOP	;7us
	NOP
	NOP
	NOP
	NOP
	NOP	;8us
	NOP
	NOP
	NOP
	NOP
	NOP	;9us
	NOP
	NOP
	NOP
	NOP
	NOP	;10us
	NOP
	NOP
	NOP
	NOP
	NOP	;11us
	NOP
	NOP
	NOP
	BTFSC	21,0	;impulz kanalov X
	BCF	5,1	;12us
	NOP
	NOP
	NOP
	NOP
	NOP	;13us
	NOP
	NOP
	NOP
	NOP
	NOP	;14us
	NOP
	NOP
	NOP
	NOP
	NOP	;15us
	NOP
	BSF	5,1	;12+3.4=15.4us
	NOP
	NOP
	NOP	;16us
	NOP
	NOP
	NOP
	NOP
	NOP	;17us
	NOP
	NOP
	NOP
	NOP
	NOP	;18us
	NOP
	NOP
	NOP
	NOP
	NOP	;19us
	NOP
	NOP
	NOP
	NOP
	NOP	;20us
	NOP
	NOP
	NOP
	NOP
	NOP	;21us
	NOP
	NOP
	NOP
	NOP
	NOP	;22us
	NOP
	NOP
	NOP
	NOP
	NOP	;23us
	NOP
	NOP
	NOP
	NOP
	NOP	;24us
	NOP
	NOP
	NOP
	NOP
	NOP	;25us
	NOP
	NOP
	NOP
	NOP
	NOP	;26us
	NOP
	NOP
	NOP
	NOP
	NOP	;27us
	NOP
	NOP
	NOP
	NOP
	NOP	;28us
	NOP
	NOP
	NOP
	NOP
	NOP	;29us
	NOP
	NOP
	NOP
	BTFSC	21,1	;impulz odgovora Y
	BCF	5,1	;30us
	NOP
	NOP
	NOP
	NOP
	NOP	;31us
	NOP
	NOP
	NOP
	NOP
	NOP	;32us
	NOP
	NOP
	NOP
	NOP
	NOP	;33us
	NOP
	BSF	5,1	;30+3.4=33.4us
	NOP
	NOP
	NOP	;34us
	NOP
	NOP
	NOP
	NOP
	NOP	;35us
	NOP
	NOP
	NOP
	BTFSC	21,2	;impulz vprasanja Y
	BCF	5,1	;36us
	NOP
	NOP
	NOP
	NOP
	NOP	;37us
	NOP
	NOP
	NOP
	NOP
	NOP	;38us
	NOP
	NOP
	NOP
	NOP
	NOP	;39us
	NOP
	BSF	5,1	;36+3.4=39.4us
	NOP
	NOP
	NOP
	NOP
	BCF	5,0	;prozenje/izklop oddaje
	RETURN

;*** Prikaz kode (21,22,23,24) ali zakasnitve (25) *** (0D)
PRIKAZ	MOVLW	0DB	;prikaz zakasnitve?
	ADDWF	4,0
	BTFSC	3,0
	GOTO	PRIKAZ1
	MOVF	21,0	;prva stevilka kode - izpis kode
	CALL	LCDDEK
	MOVWF	10
	MOVF	22,0	;druga stevilka kode
	CALL	LCDDEK
	MOVWF	11
	MOVF	23,0	;tretja stevilka kode
	CALL	LCDDEK
	MOVWF	12
	MOVF	24,0	;cetrta stevilka kode
	CALL	LCDDEK
	MOVWF	13
	BTFSS	0E,4	;utripanje znaka, ki se ga nastavlja gor/dol?
	RETURN
	MOVLW	21
	SUBWF	4,0	;prvi znak?
	BTFSC	3,2
	CLRF	10
	ADDLW	0FF	;drugi znak?
	BTFSC	3,2
	CLRF	11
	ADDLW	0FF	;tretji znak?
	BTFSC	3,2
	CLRF	12
	ADDLW	0FF	;cetrti znak?
	BTFSC	3,2
	CLRF	13
	RETURN
PRIKAZ1	MOVF	25,0	;izpis zakasnitve oddaje (25)
	MOVWF	0D
	DECF	0D,1
	CLRF	10
	MOVLW	64	;izracunaj in prikazi stotice
	CLRF	0C
PRIKAZ2	INCF	0C,1
	SUBWF	0D,1
	BTFSC	3,0
	GOTO	PRIKAZ2
	ADDWF	0D,1
	DECF	0C,0
	CALL	LCDDEK
	MOVWF	11
	MOVLW	0A	;izracunaj in prikazi desetice
	CLRF	0C
PRIKAZ3	INCF	0C,1
	SUBWF	0D,1
	BTFSC	3,0
	GOTO	PRIKAZ3
	ADDWF	0D,1
	DECF	0C,0
	CALL	LCDDEK
	MOVWF	12
	MOVF	0D,0	;izracunaj in prikazi enice
	CALL	LCDDEK
	MOVWF	13
	RETURN

;*** Preveri meje kazalca in kode *** (4,21,22,23,24,25)
MEJE	MOVLW	0E0	;spodnja meja kazalca
	ADDWF	4,1
	BTFSS	3,0
	CLRF	4
	SUBWF	4,1
	MOVLW	0DB	;gornja meja kazalca
	ADDWF	4,1
	BTFSC	3,0
	CLRF	4
	SUBWF	4,1
	MOVLW	7	;meje prve stevilke
	ANDWF	21,1
	MOVLW	7	;meje druge stevilke
	ANDWF	22,1
	MOVLW	3	;meje tretje stevilke
	ANDWF	23,1
	MOVLW	1	;meje cetrte stevilke
	ANDWF	24,1
	RETURN		;zakasnitev oddaje nima meje (0-255)

;*** Precitaj in izvedi ukaz s tipk *** (0F)
TIPKE	COMF	6,0	;precitaj RB4-7
	ANDLW	0F0
	MOVF	0F,1	;prejsnje stanje vse tipke odpuscene?
	MOVWF	0F
	BTFSS	3,2
	RETURN
	BTFSS	0F,5	;Ukaz DOL?
	GOTO	TIPKE1
	DECF	0,1
	RETURN
TIPKE1	BTFSS	0F,6	;Ukaz GOR?
	GOTO	TIPKE2
	INCF	0,1
	RETURN
TIPKE2	BTFSS	0F,4	;Ukaz DESNO?
	GOTO	TIPKE3
	INCF	4,1
	RETURN
TIPKE3	BTFSS	0F,7	;Ukaz LEVO?
	RETURN
	DECF	4,1
	RETURN

;*** Pretvori stevilko v 7-segmentni zapis *** (0C)
LCDDEK	MOVWF	0C
	INCF	0C,1
	MOVLW	7E	;stevilka 0
	DECF	0C,1
	BTFSC	3,2
	RETURN
	MOVLW	12	;stevilka 1
	DECF	0C,1
	BTFSC	3,2
	RETURN
	MOVLW	0BC	;stevilka 2
	DECF	0C,1
	BTFSC	3,2
	RETURN
	MOVLW	0B6	;stevilka 3
	DECF	0C,1
	BTFSC	3,2
	RETURN
	MOVLW	0D2	;stevilka 4
	DECF	0C,1
	BTFSC	3,2
	RETURN
	MOVLW	0E6	;stevilka 5
	DECF	0C,1
	BTFSC	3,2
	RETURN
	MOVLW	0EE	;stevilka 6
	DECF	0C,1
	BTFSC	3,2
	RETURN
	MOVLW	32	;stevilka 7
	DECF	0C,1
	BTFSC	3,2
	RETURN
	MOVLW	0FE	;stevilka 8
	DECF	0C,1
	BTFSC	3,2
	RETURN
	MOVLW	0F6	;stevilka 9
	DECF	0C,1
	BTFSC	3,2
	RETURN
	MOVLW	0FA	;crka A
	DECF	0C,1
	BTFSC	3,2
	RETURN
	MOVLW	0CE	;crka b
	DECF	0C,1
	BTFSC	3,2
	RETURN
	MOVLW	6C	;crka C
	DECF	0C,1
	BTFSC	3,2
	RETURN
	MOVLW	9E	;crka d
	DECF	0C,1
	BTFSC	3,2
	RETURN
	MOVLW	0EC	;crka E
	DECF	0C,1
	BTFSC	3,2
	RETURN
	MOVLW	0E8	;crka F
	DECF	0C,1
	BTFSC	3,2
	RETURN
	MOVLW	0A4	;crtice za napako
	RETURN

;*** Postavi podatke (10,11,12,13) v LCD pomikalni register *** (0D,0E)
LCDDAT	MOVF	13,0	;(13) v pomikalni register
	MOVWF	0D
	BTFSS	0E,0
	COMF	0D,1
	CALL	LCDBIT
	MOVF	12,0	;(12) v pomikalni register
	MOVWF	0D
	BTFSS	0E,0
	COMF	0D,1
	CALL	LCDBIT
	MOVF	11,0	;(11) v pomikalni register
	MOVWF	0D
	BTFSS	0E,0
	COMF	0D,1
	CALL	LCDBIT
	MOVF	10,0	;(10) v pomikalni register
	MOVWF	0D
	BTFSS	0E,0
	COMF	0D,1
	CALL	LCDBIT
	INCF	0E,1	;obrni fazo za BACKPLANE=RB0
	BCF	5,2
	BTFSC	0E,0
	BSF	5,2
	BSF	6,1	;impulz STROBE=RB1
	NOP
	BCF	6,1
	RETURN

;*** Poslji bajt (0D) v LCD pomikalni register *** (0C)
LCDBIT	MOVLW	8	;zanka po 8 bitih
	MOVWF	0C
LCDBIT1	BCF	6,2	;postavi niclo DATA=RB2
	RLF	0D,1	;zavrti register
	BTFSC	3,0
	BSF	6,2	;postavi enico DATA=RB2
	NOP
	BSF	6,3	;taktni impulz CLOCK=RB3
	NOP
	BCF	6,3
	DECFSZ	0C,1	;stevec zanke po 8 bitih
	GOTO	LCDBIT1
	RETURN

;*** Dvojna zanka cakanja 10ms *** (0D)
CAKAJ	MOVLW   96
	MOVWF   0D
CAKAJ1	CALL    CAKAJ2
	DECF    0D,1
	BTFSS   3,2
	GOTO    CAKAJ1
	RETURN
CAKAJ2  MOVLW   3C
CAKAJ3  NOP
	ADDLW   0FF
	BTFSS   3,2
	GOTO    CAKAJ3
	RETURN

;*** Nastavi konfiguracijo PICa 16F84 ***
	ORG	2007	;naslov konfiguracijske besede
	DW	1A	;CP=OFF, PWRTE=OFF, WDT=OFF, OSC=HS

;*** Nastavi zacetne vrednosti EEPROMa ***
	ORG	2100
	DW	2	;MODE A
	DW	7	;frekvenca 7
	DW	2	;izvor T0IE
	DW	0	;ATC
	DW	8	;zakasnitev 7us
	END

;*** Konec programa ***
	